package Reaction::UI::ViewPort::Collection::Grid::Member::WithActions;

use Reaction::Class;
use namespace::clean -except => [qw(meta)];

extends 'Reaction::UI::ViewPort::Collection::Grid::Member';

#TO BE REMOVED

1;
